@@ -18,6 +18,8 @@ import socket
 
 from napalm import get_network_driver
 from napalm.base.exceptions import ConnectionException, CommandErrorException
+import netaddr
+from netaddr.core import AddrFormatError
 
 from django.conf import settings
 from django.utils.text import slugify
@@ -48,6 +50,7 @@ class OnboardException(Exception):
         "fail-connect",  # device is unreachable at IP:PORT
         "fail-execute",  # unable to execute device/API command
         "fail-login",  # bad username/password
+        "fail-dns",  # failed to get IP address from name resolution
         "fail-general",  # other error
     )
 
@@ -122,7 +125,25 @@ class NetdevKeeper:
             raise OnboardException(reason="fail-connect", message=f"ERROR device unreachable: {ip_addr}:{port}")
 
     @staticmethod
-    def guess_netmiko_device_type(**kwargs):
+    def check_netmiko_conversion(guessed_device_type, platform_map=None):
+        """Method to convert Netmiko device type into the mapped type if defined in the settings file.
+
+        Args:
+            guessed_device_type (string): Netmiko device type guessed platform
+            test_platform_map (dict): Platform Map for use in testing
+
+        Returns:
+            string: Platform name
+        """
+        # If this is defined, process the mapping
+        if platform_map:
+            # Attempt to get a mapped slug. If there is no slug, return the guessed_device_type as the slug
+            return platform_map.get(guessed_device_type, guessed_device_type)
+
+        # There is no mapping configured, return what was brought in
+        return guessed_device_type
+
+    def guess_netmiko_device_type(self, **kwargs):
         """Guess the device type of host, based on Netmiko."""
         guessed_device_type = None
 
@@ -154,7 +175,8 @@ class NetdevKeeper:
 
         logging.info("INFO device type is %s", guessed_device_type)
 
-        return guessed_device_type
+        # Get the platform map from the PLUGIN SETTINGS, Return the result of doing a check_netmiko_conversion
+        return self.check_netmiko_conversion(guessed_device_type, platform_map=PLUGIN_SETTINGS.get("platform_map", {}))
 
     def get_platform_slug(self):
         """Get platform slug in netmiko format (ie cisco_ios, cisco_xr etc)."""
@@ -170,17 +192,30 @@ class NetdevKeeper:
         return platform_slug
 
     @staticmethod
-    def get_platform_object_from_netbox(platform_slug):
+    def get_platform_object_from_netbox(
+        platform_slug, create_platform_if_missing=PLUGIN_SETTINGS["create_platform_if_missing"]
+    ):
         """Get platform object from NetBox filtered by platform_slug.
 
+        Args:
+            platform_slug (string): slug of a platform object present in NetBox, object will be created if not present
+            and create_platform_if_missing is enabled
+
+        Return:
+            dcim.models.Platform object
+
+        Raises:
+            OnboardException
+
         Lookup is performed based on the object's slug field (not the name field)
         """
         try:
+            # Get the platform from the NetBox DB
             platform = Platform.objects.get(slug=platform_slug)
             logging.info("PLATFORM: found in NetBox %s", platform_slug)
         except Platform.DoesNotExist:
 
-            if not PLUGIN_SETTINGS["create_platform_if_missing"]:
+            if not create_platform_if_missing:
                 raise OnboardException(
                     reason="fail-general", message=f"ERROR platform not found in NetBox: {platform_slug}"
                 )
@@ -196,8 +231,54 @@ class NetdevKeeper:
             )
             platform.save()
 
+        else:
+            if not platform.napalm_driver:
+                raise OnboardException(
+                    reason="fail-general", message=f"ERROR platform is missing the NAPALM Driver: {platform_slug}",
+                )
+
         return platform
 
+    def check_ip(self):
+        """Method to check if the IP address form field was an IP address.
+
+        If it is a DNS name, attempt to resolve the DNS address and assign the IP address to the
+        name.
+
+        Returns:
+            (bool): True if the IP address is an IP address, or a DNS entry was found and
+                    reassignment of the ot.ip_address was done.
+                    False if unable to find a device IP (error)
+
+        Raises:
+          OnboardException("fail-general"):
+            When a prefix was entered for an IP address
+          OnboardException("fail-dns"):
+            When a Name lookup via DNS fails to resolve an IP address
+        """
+        try:
+            # Assign checked_ip to None for error handling
+            # If successful, this is an IP address and can pass
+            checked_ip = netaddr.IPAddress(self.ot.ip_address)
+            return True
+        # Catch when someone has put in a prefix address, raise an exception
+        except ValueError:
+            raise OnboardException(
+                reason="fail-general", message=f"ERROR appears a prefix was entered: {self.ot.ip_address}"
+            )
+        # An AddrFormatError exception means that there is not an IP address in the field, and should continue on
+        except AddrFormatError:
+            try:
+                # Do a lookup of name to get the IP address to connect to
+                checked_ip = socket.gethostbyname(self.ot.ip_address)
+                self.ot.ip_address = checked_ip
+                return True
+            except socket.gaierror:
+                # DNS Lookup has failed, Raise an exception for unable to complete DNS lookup
+                raise OnboardException(
+                    reason="fail-dns", message=f"ERROR failed to complete DNS lookup: {self.ot.ip_address}"
+                )
+
     def get_required_info(
         self,
         default_mgmt_if=PLUGIN_SETTINGS["default_management_interface"],
@@ -215,6 +296,8 @@ class NetdevKeeper:
           OnboardException('fail-general'):
             Any other unexpected device comms failure.
         """
+        # Check to see if the IP address entered was an IP address or a DNS entry, get the IP address
+        self.check_ip()
         self.check_reachability()
         mgmt_ipaddr = self.ot.ip_address
 
@@ -474,5 +557,6 @@ class NetboxKeeper:
         self.ensure_device_type()
         self.ensure_device_role()
         self.ensure_device_instance()
-        self.ensure_interface()
-        self.ensure_primary_ip()
+        if PLUGIN_SETTINGS["create_management_interface_if_missing"]:
+            self.ensure_interface()
+            self.ensure_primary_ip()
